#!/bin/python3

#
#       ███╗   ██╗██╗██╗  ██╗ █████╗ ██████╗ ███████╗
#       ████╗  ██║██║██║  ██║██╔══██╗██╔══██╗██╔════╝
#       ██╔██╗ ██║██║███████║███████║██████╔╝███████╗
#       ██║╚██╗██║██║██╔══██║██╔══██║██╔══██╗╚════██║
#       ██║ ╚████║██║██║  ██║██║  ██║██║  ██║███████║
#       ╚═╝  ╚═══╝╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚══════╝
#       DRAFTED BY [https://nihar.page] ON 18-04-2021.
#       SOURCE [creator.py] LAST MODIFIED ON 20-04-2021.
#


from os import mkdir, path
from datetime import datetime


def createConfig(pidgeyName):
    with open(path.join(pidgeyName,'config.yml'),'w') as f:
        conf_data = (
            """# Mandatory Configuration
title : Demo pidgeotto website
url : yourdomain.tld
note_template : template/note_template.html   # Template of note page
home_template : template/home_template.html   # Template of home page
feed_template : template/feed_template.xml   # Template of feed/rss page
home_path : public  # After built, static files will be generated here
content_path : content # All post notes markdown location
resource_path : static # All css, js, image location
home_md : content/home.md # Homepage content
header_md : content/header.md # Header content
footer_md : content/footer.md # Footer content
no_of_post_home : 10 # No of note in homepage
archive_md : content/archive.md # Archive page content

# Optional Configuration, Add your own configs here
site-title : Demo Pidgeotto
css : demo.css
desc : Write anything that human and machine can understand.
mail : some@mail.com
            """)
        f.write(conf_data)
        print('config.yml created.')


def createTemplates(pidgeyName):
    with open(path.join(pidgeyName,'template','home_template.html'),'w') as f:
        conf_data = (
        """<!DOCTYPE html>
<html lang="en-IN" data-theme="dark">

<head>
<meta charset="utf-8">
<meta http-equiv="X-UA-Compatible" content="IE=edge">
<meta name="viewport" content="width=device-width, initial-scale=1">
<meta name="description" content=" {{ config.get('desc') }}">
<title>{{ config.get('site-title') }}</title>
<link rel="stylesheet" type="text/css" media="screen" href="{{ config.get('css') }}" />
<link rel="alternate" type="application/atom+xml" title="Recent blog posts" href="/rss.xml">
</head>

<body>
<header>
<h1><a href="/">{% filter lower %} {{ title }} {% endfilter %}</a></h1>
{{ header }}
</header>
<section>
{{article}}
<ul>
{% for post in posts %}{% if post.showInHome is undefined%}
<li>{{ post.date.strftime('%d %m %Y') }} ; <a href="{{ post.url }}">{% filter lower %} {{ post.title }} {% endfilter %}</a></li>
{% endif %}{% endfor %}
{% if nextpage %} <a href="{{ nextpage }}">Older Notes >> </a> {% endif %}
</ul>
</section>
<footer>
{{ footer }}
</footer>
</body>
</html>
            """)
        f.write(conf_data)
        print('home_template created.')

    with open(path.join(pidgeyName,'template','note_template.html'),'w') as f:
        conf_data = (
            """<!DOCTYPE html>
<html lang="en" data-theme="dark">
<head>
<meta charset="utf-8">

<title>{{ post_title }}</title>

<meta http-equiv="X-UA-Compatible" content="IE=edge">
<meta name="viewport" content="width=device-width, initial-scale=1">
<meta name="description" content=" {{ post_subtitle }}">

<link rel="stylesheet" type="text/css" media="screen" href="{{ config.get('css') }}" />
<link rel="alternate" type="application/atom+xml" title="Recent blog posts" href="/rss.xml">
{% if metad %}  {{ metad }} {% endif %}
</head>

<body>
<header>
<h1>{{ post_title }} </h1>  
</header>
<article>
{{ article }}
{% if date %} 
<p># Posted on <time>{{ date.strftime('%d %b %Y') }}.</time></p>
{% endif %}
</article>
<footer>
{{ footer }}
</footer>
</body>
</html>
            """)
        f.write(conf_data)
        print('note template created.')

    with open(path.join(pidgeyName,'template','feed_template.xml'),'w') as f:
        conf_data = (
            """<?xml version="1.0" encoding="UTF-8"?>
<rss version="2.0"
	xmlns:content="http://purl.org/rss/1.0/modules/content/"
	xmlns:wfw="http://wellformedweb.org/CommentAPI/"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:atom="http://www.w3.org/2005/Atom"
	xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
    xmlns:slash="http://purl.org/rss/1.0/modules/slash/">

    <channel>
        <title>{{title}}</title>
        <atom:link href="{{ url }}" rel="self" type="application/rss+xml" />
        <link>{{ url }}</link>
	    <description>{{ subtitle }}</description>
        <lastBuildDate>{{ last_date.strftime('%a, %d %b %Y %H:%M:%S %z') }}</lastBuildDate>
	    <language>en-US</language>
	    <sy:updatePeriod>weekly</sy:updatePeriod>
	    <sy:updateFrequency>1</sy:updateFrequency>
        
        <image>
            <url>{{url}}{{ avatar }}</url>
	        <title>{{ title }}</title>
            <link>{{ url }}</link>
	        <width>32</width>
	        <height>32</height>
        </image>

		{% for post in posts %}{% if post.showInHome is undefined%}
        <item>
            <title>{{ post.title }}</title>
            <link>{{ url}}{{ post.url }}</link>
            <pubDate>{{ post.date.strftime('%a, %d %b %Y %H:%M:%S %z') }}</pubDate>
            <guid isPermaLink="false">{{ url }}{{ post.url }}</guid>
			<description><![CDATA[{{ post.subtitle }} - {{ post.note }} ]]></description>
        </item>
		{% endif %}{% endfor %}

    </channel>
</rss>
            """)
        f.write(conf_data)
        print('rss template created.')
        f.close()


def createContent(pidgeyName):
    with open(path.join(pidgeyName,'content','header.md'),'w') as f:
        conf_data = (
            """*   [Home](/)
*   [Other Links](/)
*   [Edit content/header.md](/)
            """)
        f.write(conf_data)
        print('header created.')

    with open(path.join(pidgeyName,'content','footer.md'),'w') as f:
        conf_data = (
            """*   [Home](/)
*   [RSS](/)
*   [Edit content/footer.md](/)
*   powered by [Pidgeotto](/)
            """)
        f.write(conf_data)
        print('footer created.')

    with open(path.join(pidgeyName,'content','home.md'),'w') as f:
        conf_data = (
            """Welcome to Pidgeotto.
<br> This is a sample homepage which can be edited at /content/home.md

** Pidgeotto links: **
*   [Documentation](/).
*   [Github Source](/).
        """)
        f.write(conf_data)
        print('footer created.')
    
    with open(path.join(pidgeyName,'content','archive.md'),'w') as f:
        conf_data = (
            """# Archives
        """)
        f.write(conf_data)
        print('archive created.')

    with open(path.join(pidgeyName,'content','note','blog1.md'),'w') as f:
        conf_data = (
            """---
title : "Sample Blog."
subtitle : "Sample subtitle"
date : {t}
---

# This is a sample note page which can be edited/renamed at /content/note/blog1.md
# This is heading 1

## This is heading 2

### This is heading 3

#### This is heading 4

##### This is heading 5

###### This is heading 6

**Bold**

*italic*

* list
* list

1. ordered list
2. ordered list

[link](/)

Sample paragraph is written like this with lorem ipsum. Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.
        """).format(t=datetime.now().strftime("%Y-%m-%d"))
        f.write(conf_data)
        print('sample blog created.')


def createPidgey(pidgeyName):
    try:
        mkdir(pidgeyName)
        print("pidgey " + pidgeyName + " created.")
        mkdir(path.join(pidgeyName,'template'))
        mkdir(path.join(pidgeyName,'static'))
        mkdir(path.join(pidgeyName,'content'))
        mkdir(path.join(pidgeyName,'content','note'))
        createConfig(pidgeyName)
        createTemplates(pidgeyName)
        createContent(pidgeyName)
    except:
        print("while generating pidgey "+ pidgeyName + ", some issue occured.")
        print("This can be due to \n\t1. Pidgey already exists.\n\t2. Memory issue.\n\t3. Invalid input.\n\t4. If none of these, report to developer.")

def createNote(noteName, showInHome=True):
    with open(path.join('content','note',noteName),'w') as f:
        conf_data = (
        """---
title : "Sample Blog."
subtitle : "Sample subtitle"
showInHome : {s}
date : {t}
---
        """).format(t=datetime.now().strftime("%Y-%m-%d"), s=showInHome)
        f.write(conf_data)
        print(noteName+" is created at content/note/"+noteName )




#!/bin/python

#   
#       ███╗   ██╗██╗██╗  ██╗ █████╗ ██████╗ ███████╗
#       ████╗  ██║██║██║  ██║██╔══██╗██╔══██╗██╔════╝
#       ██╔██╗ ██║██║███████║███████║██████╔╝███████╗
#       ██║╚██╗██║██║██╔══██║██╔══██║██╔══██╗╚════██║
#       ██║ ╚████║██║██║  ██║██║  ██║██║  ██║███████║
#       ╚═╝  ╚═══╝╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚══════╝
#       DRAFTED BY [https://nihar.page] ON 10-04-2021.
#       SOURCE [builder.py] LAST MODIFIED ON 20-04-2021.
#

# External imports
from shutil import copytree, rmtree, copy
from os import path, makedirs
from glob import glob
from yaml import safe_load
from jinja2 import Environment, FileSystemLoader
from markdown2 import markdown

def buildPidgey():
    # Create Page
    def create_page(template,post_detail,md,filename):
        post_template = Environment(loader=FileSystemLoader(searchpath='./')).get_template(template)
        post_title = title
        post_date = post_data = posts_list = last_date = nextpage = post_meta = post_subtitle = ""
        post_path =  home_path
        if filename=="index.html":
            post_file = filename
            posts_list = posts[0:no_of_post_home]
            nextpage = "archive.html" if (len(posts)>no_of_post_home) else ""
        elif filename=="archive.html":
            post_file = filename
            posts_list = posts[no_of_post_home:]
        elif filename.endswith(".xml") :
            post_file = filename
            posts_list = posts
            last_date = posts_list[0].get('date')
        elif post_detail == None :
            post_file = filename.replace('.md','.html')
        else:
            post_title = post_detail.get("title")
            post_subtitle = post_detail.get("subtitle")
            post_date = post_detail.get("date")
            post_meta = post_detail.get("meta")
            post_data = filename.split('/')
            post_path = path.join(home_path)
            post_file = post_data[2].replace('.md','.html')
            post_data = post_data[1]
            
            makedirs(post_path,exist_ok=True)
        with open(path.join(post_path,post_file),'w') as output_file:
            output_file.write(
                post_template.render(
                    title = title,
                    post_title = post_title,
                    post_subtitle = post_subtitle,
                    date = post_date,
                    metad = post_meta,
                    url = path.join(url,post_file),
                    article = markdown(md),
                    posts = posts_list,
                    home = home_md,
                    header = markdown(readmd(header_md)),
                    footer = markdown(readmd(footer_md)),
                    nextpage = nextpage,
                    last_date = last_date,
                    config=config
                )
            )
        output_file.close()
        return post_file
    
    
    # Markdown file to string
    def readmd(md):
        with open(md,'r') as data:
            return data.read()


    # PROGRAM STARTS HERE
    with open('config.yml') as conf:
        config = safe_load(conf.read())
        for key,val in config.items():
            globals()[key] = val

    # Recreated home path with resource
    if path.exists(home_path):
        rmtree(home_path)
    copytree(resource_path, home_path)
    
    #Create all pages from content/note
    posts = []
    for note in glob(path.join(content_path,"note","*.md")):
        yaml_lines, ym, md = [],'',''
        with open(note) as infile:
            for s in infile:
                if s.startswith('---'):
                    for s in infile:
                        if s.startswith('---'):
                            break;
                        else:
                            yaml_lines.append(s)
                    ym = ''.join(yaml_lines)
                    md = ''.join(infile)
                    break;
        post_detail=safe_load(ym)
        if (post_detail is not None):
            post_url = create_page(note_template,post_detail,md,note)
            ymd = post_detail
            ymd.update({'url' : '/'+post_url})
            ymd.update({'note' : markdown(md)})
            posts += [ymd]
    
    #Sort posts based on date in descending order
    posts= sorted(posts, key=lambda post :  post['date'], reverse=True)
    
    # Other pages are created here
    create_page(home_template,None,readmd(home_md),"index.html")
    create_page(home_template,None,readmd(archive_md),"archive.html")
    create_page(feed_template,None,readmd(home_md),"rss.xml")
    print("Public directory created containing all static pages")



from setuptools import setup
from pidgey import __version__

with open('README.md') as desc:
   desc_md = desc.read()

setup(
        name='pidgeotto',
        version=__version__,
        description='Yet another static blog generator.',
        long_description=desc_md,
        long_description_content_type='text/markdown',
        author='nihar',
        author_email='<mail@nihars.com>',
        url='https://github.com/niharokz/pidgeotto',
        license='GPLv3+',
        packages=['pidgey'],
        install_requires=[
            'markdown2', 'Jinja2', 'pyyaml',
            ],
        keywords=['python','jinja2','pyyam','markdown2','static','site'],
        classifiers=[
            "Development Status :: 1 - Planning",
            "Intended Audience :: Developers",
            "Programming Language :: Python :: 3",
            "Operating System :: Unix",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
            ],
        extra_require={
            "dev": [
                "pytest>=3.7",
                ],
            },
        entry_points={
            'console_scripts': [
                'pidgey= pidgey.pidgey:main',
                ]
            },
)



#!/bin/python

#   
#       ███╗   ██╗██╗██╗  ██╗ █████╗ ██████╗ ███████╗
#       ████╗  ██║██║██║  ██║██╔══██╗██╔══██╗██╔════╝
#       ██╔██╗ ██║██║███████║███████║██████╔╝███████╗
#       ██║╚██╗██║██║██╔══██║██╔══██║██╔══██╗╚════██║
#       ██║ ╚████║██║██║  ██║██║  ██║██║  ██║███████║
#       ╚═╝  ╚═══╝╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚══════╝
#       DRAFTED BY [https://nihar.page] ON 10-04-2021.
#       SOURCE [pidgey.py] LAST MODIFIED ON 20-04-2021.
#

# External imports
from pidgey import creator,builder,__version__
from argparse import ArgumentParser

    
def main():
    parser = ArgumentParser(prog='pidgey', description='Yet another simple static site generator.')
    parser.add_argument("-v", "--version", action="version", version="{version}".format(version=__version__) )
    subparsers = parser.add_subparsers(dest='type', help='command help', required=True)
    parser_init = subparsers.add_parser('init', help='start new pidgeotto').add_argument("name")
    parser_new = subparsers.add_parser('new', help='create new note/post/page').add_argument("name")
    parser_build = subparsers.add_parser('build', help='build the pidgeotto')
    args = parser.parse_args()
    if args.type == "init" and args.name:
        creator.createPidgey(args.name)
    elif args.type == "new" and args.name:
        creator.createNote(args.name)
    elif args.type == 'build':
        builder.buildPidgey()
    else: 
        parser.print_help()
